#include <E-mail.h>
#include <String.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


#define BUFFER_SIZE 256


void print_usage(int argc, char **argv)
{
   printf("USAGE: %s [-s subject] [recipient]\n", argv[0]);
   exit(0);
}


int main(int argc, char *argv[])
{
   if (argc > 4) print_usage(argc, argv);

   BString subject = NULL;
   BString recipient = NULL;

   if (argc > 2) {
      if (strcmp(argv[1], "-s") == 0) {
	 subject = argv[2];
      } else {
	 print_usage(argc, argv);
      }

      if (argc == 4) recipient = argv[3];
   } else if (argc == 2) {
      if (*argv[1] == '-') {
	 print_usage(argc, argv);
      } else {
	 recipient = argv[1];
      }
   }

   bool atty = isatty(fileno(stdin));
   char buffer[BUFFER_SIZE];

   if (atty && (recipient == NULL)) {
      printf("To: ");
      fgets(buffer, BUFFER_SIZE, stdin);
      /* remove the new line character */
      buffer[strlen(buffer)-1] = '\0';
      recipient = buffer;
   } else if (recipient == NULL) {
      print_usage(argc, argv);
   }   

   if (atty && (subject == NULL)) {
      printf("Subject: ");
      fgets(buffer, BUFFER_SIZE, stdin);
      /* remove the new line character */
      buffer[strlen(buffer)-1] = '\0';
      subject = buffer;
   } else if (subject == NULL) {
      subject = "No Subject";
   }

   BMailMessage *mail = new BMailMessage();
   mail->AddHeaderField(B_MAIL_TO, recipient.String());
   mail->AddHeaderField(B_MAIL_SUBJECT, subject.String());

   if (atty) {
      /* Let the user create the mail message. */
      printf("Type a period (`.') on a line by itself to send this message.\n");
      do {
	 fgets(buffer, BUFFER_SIZE, stdin);
	 if ((buffer[0] == '.') && (buffer[1] == '\n'))
	    break;
	 mail->AddContent(buffer, strlen(buffer));
      } while (1);	
   } else {     
      /* Create a mail message from the text that was piped in. */
      do {
	 fgets(buffer, BUFFER_SIZE, stdin);
	 /* We may reach the end of the file on the last line of
	    input, but we still want that line to be sent. */
	 if (!feof(stdin) 
	     || (feof(stdin) && (buffer[strlen(buffer)-1] != '\n'))) {
	    mail->AddContent(buffer, strlen(buffer));
	 }
      } while (!feof(stdin));
   }

   /* Send the email message without waiting. */	
   mail->Send(true);
   
   delete mail;
}
